import {
  Button,
  LinearProgress,
  Grid,
  Typography,
  Tooltip
} from "@mui/material";
import { useState, useEffect, useCallback } from "react";
import { ConfirmationDialog } from "@ess-ics/ce-ui-common";
import Alert from "@mui/material/Alert";
import { initRequestParams } from "../../common/Helper";
import AccessControl from "../../auth/AccessControl";
import { useNavigate } from "react-router-dom";
import { useStartOperationMutation } from "../../../store/deployApi";
import { getErrorMessage } from "../../common/Alerts/ApiAlertError";

export function IOCService({
  ioc,
  currentCommand,
  getOperations,
  buttonDisabled,
  setButtonDisabled,
  jobLazyParams
}) {
  const [error, setError] = useState();
  const navigate = useNavigate();
  const [inProgress, setInProgress] = useState(false);
  const [startModalOpen, setStartModalOpen] = useState(false);
  const [stopModalOpen, setStopModalOpen] = useState(false);
  const [command, setCommand] = useState(null);

  const [startIOC, { error: startJobError, data: startJob }] =
    useStartOperationMutation();
  const [stopIOC, { error: stopJobError, data: stopJob }] =
    useStartOperationMutation();

  useEffect(() => {
    setError(startJobError);
  }, [startJobError]);

  useEffect(() => {
    setError(stopJobError);
  }, [stopJobError]);

  useEffect(() => {
    if (startJob && (!command || command.id !== startJob.id)) {
      navigate(`/jobs/${startJob.id}`);
      setCommand(startJob);
    } else if (stopJob && (!command || command.id !== stopJob.id)) {
      navigate(`/jobs/${stopJob.id}`);
      setCommand(stopJob);
    } else if (currentCommand) {
      setCommand(currentCommand);
    }
  }, [startJob, stopJob, command, currentCommand, navigate]);

  const resetUI = useCallback(() => {
    setCommand(null);
    setError(null);
  }, []);

  const start = useCallback(() => {
    resetUI();
    setInProgress(true);
    setButtonDisabled(true);

    startIOC(
      {
        ioc_id: ioc.id
      },
      {
        requestBody: {
          type: "START"
        }
      }
    );

    let requestParams = initRequestParams(jobLazyParams);

    requestParams.deployment_id = ioc.activeDeployment?.id;
    getOperations(requestParams);
  }, [
    getOperations,
    ioc.activeDeployment?.id,
    jobLazyParams,
    resetUI,
    setButtonDisabled,
    startIOC
  ]);

  const stop = useCallback(() => {
    resetUI();
    setInProgress(true);
    setButtonDisabled(true);

    stopIOC(
      {
        ioc_id: ioc.id
      },
      {
        requestBody: {
          type: "STOP"
        }
      }
    );

    let requestParams = initRequestParams(jobLazyParams);

    requestParams.deployment_id = ioc.activeDeployment?.id;
    getOperations(requestParams);
  }, [
    getOperations,
    ioc.activeDeployment?.id,
    jobLazyParams,
    resetUI,
    setButtonDisabled,
    stopIOC
  ]);

  const onStartModalClose = useCallback(() => {
    setStartModalOpen(false);
  }, [setStartModalOpen]);

  const onStartModalConfirm = useCallback(() => {
    start();
  }, [start]);

  const onStopModalClose = useCallback(() => {
    setStopModalOpen(false);
  }, [setStopModalOpen]);

  const onStopModalConfirm = useCallback(() => {
    stop();
  }, [stop]);

  let disabledStartButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStartButtonTitle =
      "There is an ongoing operation, you can not Start the IOC right now";
  }

  let disabledStopButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStopButtonTitle =
      "There is an ongoing operation, you can not Stop the IOC right now";
  }

  return (
    <AccessControl
      allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
      renderNoAccess={() => "You are not authorized to control this IOC"}
    >
      <ConfirmationDialog
        title={
          <Typography
            variant="h2"
            marginY={1}
          >
            Start IOC
          </Typography>
        }
        content={
          <>
            <Typography component="span">
              Are you sure want to start{" "}
              <Typography
                component="span"
                fontFamily="monospace"
              >
                {ioc.namingName}
              </Typography>{" "}
              ?
            </Typography>
          </>
        }
        confirmText="Start IOC"
        cancelText="Cancel"
        open={startModalOpen}
        onClose={onStartModalClose}
        onConfirm={onStartModalConfirm}
      />
      <ConfirmationDialog
        title={
          <Typography
            variant="h2"
            marginY={1}
          >
            Stop IOC
          </Typography>
        }
        content={
          <>
            <Typography component="span">
              Are you sure want to stop{" "}
              <Typography
                component="span"
                fontFamily="monospace"
              >
                {ioc.namingName}
              </Typography>{" "}
              ?
            </Typography>
          </>
        }
        confirmText="Stop IOC"
        cancelText="Cancel"
        open={stopModalOpen}
        onClose={onStopModalClose}
        onConfirm={onStopModalConfirm}
      />
      <Grid
        container
        spacing={1}
      >
        <Grid item>
          <Tooltip title={disabledStopButtonTitle}>
            <Button
              color="secondary"
              disabled={buttonDisabled}
              variant="contained"
              sx={{
                ":hover": {
                  bgcolor: "secondary.light"
                }
              }}
              onClick={() => setStopModalOpen(true)}
            >
              Stop
            </Button>
          </Tooltip>
        </Grid>
        <Grid item>
          <Tooltip title={disabledStartButtonTitle}>
            <Button
              color="essGrass"
              variant="contained"
              disabled={buttonDisabled}
              sx={{
                color: "essWhite.main" // theme.palette.primary.main
              }}
              onClick={() => setStartModalOpen(true)}
            >
              Start
            </Button>
          </Tooltip>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {error ? (
            <Alert severity="error">{getErrorMessage(error)}</Alert>
          ) : (
            inProgress && <LinearProgress color="primary" />
          )}
        </Grid>
      </Grid>
    </AccessControl>
  );
}
